<?php

/**
 * @file
 * Contains \MediaNetXStreamWrapper.
 */

/**
 * Create an instance of MediaNetXStreamWrapper.
 *
 * @code
 *  $netx = new MediaNetXStreamWrapper('netx://a/[asset-id]/s/[asset-size]/n/[asset-file-name]');
 * @endcode
 */
class MediaNetXStreamWrapper extends MediaReadOnlyStreamWrapper {

  /**
   * {@inheritdoc}
   */
  protected $parameters = array(
    // Asset ID.
    'a' => NULL,
    // @todo Category ID.
    'c' => NULL,
    // File size.
    's' => NULL,
    // File name.
    'n' => NULL,
  );

  /**
   * The string buffer to write.
   *
   * @var string
   */
  protected $tmpFile = '';

  /**
   * {@inheritdoc}
   *
   * Adds own mime types for provided content.
   */
  public static function getMimeType($uri, $mapping = NULL) {
    $mime_type = DrupalLocalStreamWrapper::getMimeType($uri, $mapping);
    $mime_parts = explode('/', $mime_type);
    switch ($mime_parts[0]) {
      case 'image':
        return 'image/netx';

      case 'video':
        return 'video/netx';

      case 'audio':
        return 'audio/netx';

    }
    return $mime_type;
  }

  /**
   * {@inheritdoc}
   */
  protected function getTarget($uri = NULL) {
    // @todo The usage is not clear, probably needs to implement and use.
    return FALSE;
  }

  /**
   * Returns a local image file path.
   *
   * Copies a file if needed.
   *
   * @return string|bool
   *   A local path, FALSE if error.
   */
  public function getLocalImagePath() {
    $local_file = $this->getAssetFilename();
    if (!file_exists($local_file)) {
      $dirname = drupal_dirname($local_file);
      file_prepare_directory($dirname, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS);

      $repository = new NetXRepository();
      try {
        $asset_image = $repository->getImageFileContent($this->getAssetID());
      }
      catch (Exception $e) {
        // @todo add some kind of logging here.
        watchdog('netx', $e->getMessage(), array(), WATCHDOG_CRITICAL);
        return FALSE;
      }

      file_unmanaged_save_data($asset_image, $local_file, TRUE);
    }
    return $local_file;
  }

  /**
   * Returns a local preview file path.
   *
   * Copies a file if needed.
   *
   * @param string $path
   *   The remote path of the file preview.
   *
   * @return string
   *   A local path.
   */
  public function getLocalPreviewPath($path) {
    $local_file = file_default_scheme() . '://media-netx' . $path;
    if (!file_exists($local_file)) {
      $dirname = drupal_dirname($local_file);
      file_prepare_directory($dirname, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS);

      $repository = new NetXRepository();
      // Copy preview image (jpg). Each asset has one.
      $asset_image = $repository->getFileContent($path);
      file_unmanaged_save_data($asset_image, $local_file, TRUE);
    }
    return $local_file;
  }

  /**
   * Calculates asset ID from uri.
   *
   * @return int
   *   Asset ID.
   */
  public function getAssetID() {
    $params = $this->get_parameters();
    return $params['a'];
  }

  /**
   * Builds local filename.
   *
   * @todo Implement scalable logic for sub-folders (use category from JSON).
   *
   * @return string
   *   A local file name.
   */
  protected function getAssetFilename() {
    $params = $this->get_parameters();
    $local_path = file_default_scheme() . '://media-netx/';
    return $local_path . $params['a'] . '/' . $params['n'];
  }

  /**
   * {@inheritdoc}
   */
  public function setUri($uri) {
    parent::setUri($uri);
    // Set canonical URI.
    if (($params = $this->get_parameters()) && isset($params['a']) && isset($params['n'])) {
      // Normalize only valid assets with file name.
      $this->uri = 'netx://' . implode('/', $this->get_parameters());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function interpolateUrl() {
    if ($parameters = $this->get_parameters()) {
      // Redirect to NetX CDN via REST.
      $repo = new NetXRepository();
      return $repo->getRemoteOriginal(
        $parameters['a'],
        $parameters['n'],
        $this->getMimeType($parameters['n'])
      );
    }
  }

  /**
   * Checks if a particular path is a directory.
   *
   * @param string $path
   *   The path to check
   *
   * @return bool
   *   TRUE when target category not found.
   */
  protected function isDirectory($path) {
    list(, $target) = explode('://', $path, 2);

    $categories = netx_get_categories();
    return (bool) _netx_find_category($target, $categories['subCategories']);
  }

  /**
   * {@inheritdoc}
   */
  public function mkdir($uri, $mode, $options) {
    return TRUE;
  }

  // @codingStandardsIgnoreStart
  // @todo Code sniffer needs lowerCamel format.
  /**
   * {@inheritdoc}
   */
  public function url_stat($path, $flags) {
    if ($path == 'netx://') {
      // Return same stat array as on public:// so drupal will think that
      // netx:// folder works and writable.
      return array (
        0 => 20,
        1 => 400174,
        2 => 040777,
        3 => 7,
        4 => 1000,
        5 => 1000,
        6 => 0,
        7 => 4096,
        8 => 1420451192,
        9 => 1420451191,
        10 => 1420451191,
        11 => 32768,
        12 => 8,
        'dev' => 20,
        'ino' => 400174,
        'mode' => 040777,
        'nlink' => 7,
        'uid' => 1000,
        'gid' => 1000,
        'rdev' => 0,
        'size' => 4096,
        'atime' => 1420451192,
        'mtime' => 1420451191,
        'ctime' => 1420451191,
        'blksize' => 32768,
        'blocks' => 8,
      );
    }

    // @todo Make sync configurable.
    $defaults = parent::url_stat($path, $flags);
    // Try to retrieve asset metadata from URI.
    $this->setUri($path);
    $params = $this->get_parameters();
    if (isset($params['a'])) {
      $defaults['size'] = $params['s'];
    }
    elseif ($this->isDirectory($path)) {
      // Suppose directory.
      $defaults['mode'] = 040777;
    }
    else {
      // FALSE for file_exists() and friends.
      // @todo Check file exists in DAM (could be slow).
      return FALSE;
    }
    return $defaults;
  }

  /**
   * {@inheritdoc}
   */
  protected function _parse_url($url) {
    $path = explode('://', $url);
    $parts = explode('/', $path[1]);
    // Image style used.
    if ($parts[0] == 'styles') {
      // Remove styles parts.
      unset($parts[0]);
      unset($parts[1]);
      unset($parts[2]);
    }
    return parent::_parse_url('netx://' . implode('/', $parts));
  }

  /**
   * {@inheritdoc}
   */
  public function dirname($uri = NULL) {
    list($scheme, $target) = explode('://', $uri, 2);
    $dirname = dirname(trim($target, '\/'));

    if ($dirname == '.') {
      $dirname = '';
    }

    return $scheme . '://' . $dirname;
  }

  /**
   * {@inheritdoc}
   */
  public function stream_open($uri, $mode, $options, &$opened_url) {
    $this->uri = $uri;
    if ($mode === 'wb') {
      // Open for writing.
      $this->tmpFile = '';
    }
    // @todo Make sure we able to write and asset exists.
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function stream_close() {
    // If we are not writing to stream just return TRUE.
    if (empty($this->tmpFile)) {
      return TRUE;
    }

    // Extract file name to find mime type.
    $file_name = drupal_basename($this->uri);
    $extension = array_pop(explode('.', $file_name));
    $mimetype = media_get_extension_mimetype($extension);

    $repo = new NetXRepository();
    $repo->uploadFileData($this->tmpFile, $file_name, $mimetype);
    // Asset import takes a time, so loop import.
    $count = 3;
    do {
      $asset_id = $repo->importAsset($file_name);
      // Give a time to process data.
      usleep(2000);
      $status = $repo->getImportStatus($asset_id);
      $count--;
    }
    while ($count && ($status == 0 || $status == 2));

    if (empty($asset_id)) {
      watchdog('netx', 'Error importing asset', WATCHDOG_ERROR);
      return FALSE;
    }

    // Save metadata to be populated in netx_file_presave().
    $uploads = &drupal_static('netx_file_presave');
    $uploads[$this->uri] = $asset_id;
    // @todo Make watchdog configurable.
    watchdog('netx', 'Imported asset %asset', array(
      '%asset' => $asset_id,
    ), WATCHDOG_INFO);
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function stream_lock($operation) {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function stream_read($count) {
    // @todo Allow read from remote file.
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function stream_write($data) {
    $this->tmpFile .= $data;
    return strlen($data);
  }

  /**
   * {@inheritdoc}
   */
  public function stream_eof() {
    // @todo Implement properly.
    return parent::stream_eof();
  }

  /**
   * {@inheritdoc}
   */
  public function stream_seek($offset, $whence) {
    // @todo Implement properly.
    return parent::stream_seek($offset, $whence);
  }

  /**
   * {@inheritdoc}
   */
  public function stream_flush() {
    // @todo Implement properly. Should send data to DAM.
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function stream_tell() {
    // @todo Implement properly.
    return parent::stream_tell();
  }

  /**
   * {@inheritdoc}
   */
  public function stream_stat() {
    // @todo Implement properly.
    return parent::stream_stat();
  }

  /**
   * {@inheritdoc}
   */
  public function chmod($mode) {
    // @todo Implement properly.
    return TRUE;
  }
  // @codingStandardsIgnoreEnd

  /**
   * {@inheritdoc}
   */
  public function getLocalPath($uri = NULL) {
    return FALSE;
  }
}
